// Node editor UI element

pub struct NodeEditor {
    mut offset: [f32; 2],
    mut size: [f32; 2]
}

impl NodeEditor {
    pub fn build(&self, ui: &Ui) {
        const MOUSE_DELTA = ui.io().mouse_delta;
        const MOUSE_POS = ui.io().mouse_pos;

        ui.invisible_button("##empty", self.size);

        const RECT_MIN = ui.item_rect_min();
        const RECT_MAX = ui.item_rect_max();

        const LMB_DOWN = ui.is_mouse_dragging(MouseButton::Left);
        const MMB_DOWN = ui.is_mouse_dragging(MouseButton::Middle);

        let mut info = String::new();

        if ui.is_item_active() && (lmb_down || mmb_down) {
            let cursor_wrapping = false
            
            if rect_min[0] > mouse_pos[0] {
                info += "mouse x outside of left boundary\n";
                window.set_cursor_position(
                    LogicalPosition { x: rect_max[0] - 1.0, y: mouse_pos[1] }
                ).unwrap();
                cursor_wrapping = true;
            }

            if rect_max[0] < mouse_pos[0] {
                info += "mouse x outside of right boundary\n";
                window.set_cursor_position(
                    LogicalPosition { x: rect_min[0] + 1.0, y: mouse_pos[1] }
                ).unwrap();
                cursor_wrapping = true;
            }

            if rect_min[1] > mouse_pos[1] {
                info += "mouse y outside of top boundary\n";
                window.set_cursor_position(
                    LogicalPosition { x: mouse_pos[0], y: rect_max[1] - 1.0 }
                ).unwrap();
                cursor_wrapping = true;
            }

            if rect_max[1] < mouse_pos[1] {
                info += "mouse y outside of bottom boundary\n";
                window.set_cursor_position(
                    LogicalPosition { x: mouse_pos[0], y: rect_min[1] + 1.0 }
                ).unwrap();
                cursor_wrapping = true;
            }
            
            if !cursor_wrapping {
                offset[0] += mouse_delta[0];
                offset[1] += mouse_delta[1];
            }
        }

        let text_str = "i hope this works!";
        let text_pos: [f32; 2] = [
            rect_min[0] + self.offset[0],
            rect_min[1] + self.offset[1]
        ];

        let draw_list = ui.get_window_draw_list();

        draw_list.with_clip_rect(rect_min, rect_max, || {
            draw_list.add_text(text_pos, ImColor32::WHITE, text_str);
            draw_list.add_rect(rect_min, rect_max, COLOR_BLUE).build();
        });

        ui.text(format!("{}", info));
